/*
 * This file is part of hp-utils.
 * Copyright (C) 2008 Michael Denk
 *
 * hp-utils is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * hp-utils is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with hp-utils; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Michael Denk
 */

#include <stdio.h>
#include <getopt.h>

#include "logger.h"
#define SELF "hp-probe"
#define VERSION "0.1.1"
#include "util.h"
#include "device.h"

static int probe()
{
	struct device *devices, *device;

	devices = device_probe_devices();

	if (devices == NULL) {
		puts("No HP devices found.");
		return 0;
	}

	for (device = devices; device != NULL; device = device->next) {
		printf("%s (%s)\n", device->model, device->uri);
	}

	return 0;
}

void help(const char *pname)
{
	printf("Usage: %s [OPTION]...\n"
			"Probe connected HP devices.\n"
			"\n"
			"  -l, --log-level=LEVEL     set log level to LEVEL\n"
#ifndef NDEBUG
			"                            {none, error, warning, debug, debug2, all}\n"
#else
			"                            {none, error, all}\n"
#endif
			"      --help                display this help and exit\n"
			"      --version             output version information and exit\n",
			pname);
}

int main(int argc, char **argv)
{
	int c;

	log_program = SELF;
	log_level = LOG_LEVEL_ERROR;

	while (1) {
		static struct option long_options[] = {
			{ "log-level",	required_argument,	0, 'l' },
			{ "help",		no_argument,		0, 250 },
			{ "version",	no_argument,		0, 251 },
			{ 0, 0, 0, 0 }
		};
		int option_index = 0;

		c = getopt_long(argc, argv, "l:", long_options, &option_index);

		if (c == -1) {
			break;
		}

		switch (c) {
			case 'l':
				if (log_parse(optarg, &log_level) != 0) {
					helpinfo(argv[0], "illegal log level `%s'", optarg);
					return 1;
				}
				break;
			case 250:
				help(argv[0]);
				return 0;
			case 251:
				print_version();
				return 0;
			case '?':
				helpinfo(argv[0], NULL);
				return 1;
		}
	}

	if (optind < argc) {
		helpinfo(argv[0], "extra operand `%s'", argv[optind]);
		return 1;
	}

	print_copy_msg();
	return probe();
}
