/*
 * Logging facility
 * Copyright (c) 2008 Michael Denk <code@michaeldenk.de>
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef _LOGGER_H_

#define _LOGGER_H_ 1

#define LOG_LEVEL_NONE		0
#define LOG_LEVEL_ERROR		10
#define LOG_LEVEL_WARNING	20
#define LOG_LEVEL_INFO		30
#define LOG_LEVEL_DEBUG		50
#define LOG_LEVEL_DEBUG2	51
#define LOG_LEVEL_ALL		99

#define LOG_LEVEL_NONE_STR		"none"
#define LOG_LEVEL_ERROR_STR		"error"
#define LOG_LEVEL_WARNING_STR	"warning"
#define LOG_LEVEL_INFO_STR		"info"
#define LOG_LEVEL_DEBUG_STR		"debug"
#define LOG_LEVEL_DEBUG2_STR	"debug2"
#define LOG_LEVEL_ALL_STR		"all"

extern int log_level;
extern const char *log_program;

int log_parse(const char *s, int *lvl);

#ifndef NDEBUG
#define log_error(fmt, args...) \
	_log(LOG_LEVEL_ERROR,	__FILE__, __LINE__, fmt, ## args)
#define log_warn(fmt, args...) \
	_log(LOG_LEVEL_WARNING,	__FILE__, __LINE__, fmt, ## args)
#define log_info(fmt, args...) \
	_log(LOG_LEVEL_INFO,	__FILE__, __LINE__, fmt, ## args)
#define log_debug(fmt, args...) \
	_log(LOG_LEVEL_DEBUG,	__FILE__, __LINE__, fmt, ## args)
#define log_debug2(fmt, args...) \
	_log(LOG_LEVEL_DEBUG2,	__FILE__, __LINE__, fmt, ## args)
void _log(int level, const char *file, int line, const char *fmt, ...);
#define log_data(buf, size) _log_data(buf, size)
#else
#define log_error(fmt, args...) _log(LOG_LEVEL_ERROR, fmt, ## args)
#define log_warn(fmt, args...)
#define log_info(fmt, args...)
#define log_debug(fmt, args...)
#define log_debug2(fmt, args...)
void _log(int level, const char *fmt, ...);
#define log_data(buf, size)
#endif

void _log_data(const char *buf, size_t size);

#endif
