/*
 * This file is part of hp-utils.
 * Copyright (C) 2008 Michael Denk
 *
 * hp-utils is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * hp-utils is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with hp-utils; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Michael Denk
 */

/* Original copyright notice of base/pml.py from HPLIP */
/* */
/* (c) Copyright 2003-2007 Hewlett-Packard Development Company, L.P. */
/* */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License, or */
/* (at your option) any later version. */
/* */
/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */
/* */
/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA */
/* */
/* Author: Don Welch */
/* */

#ifndef PML_H_
#define PML_H_ 1

#include <stdlib.h>

#define OID_BUF_SIZE	32

struct pml {
	char oid[OID_BUF_SIZE];
	int type;
};

/* Request codes */
#define GET_REQUEST 0x00
#define GET_NEXT_REQUEST 0x01
#define GET_BLOCK_REQUEST 0x02
#define GET_NEXT_BLOCK_REQUEST 0x03
#define SET_REQUEST 0x04
#define ENABLE_TRAP_REQUEST 0x05
#define DISABLE_TRAP_REQUEST 0x06
#define TRAP_REQUEST 0x07

/* Data types */
#define TYPE_MASK 0xfc
#define TYPE_OBJECT_IDENTIFIER 0x00
#define TYPE_ENUMERATION 0x04
#define TYPE_SIGNED_INTEGER 0x08
#define TYPE_REAL 0x0C
#define TYPE_STRING 0x10
#define TYPE_BINARY 0x14
#define TYPE_ERROR_CODE 0x18
#define TYPE_NULL_VALUE 0x1c
#define TYPE_COLLECTION 0x20
#define TYPE_UNKNOWN 0xff

#define PML_CLEAN_OID "1.4.1.5.1.1"
#define PML_CLEAN_TYPE TYPE_ENUMERATION
#define PML_CLEAN_CLEAN			100
#define PML_CLEAN_PRIME			200
#define PML_CLEAN_WIPE_AND_SPIT	300

#define PML_DATE_AND_TIME_OID "1.1.2.17"
#define PML_DATE_AND_TIME_TYPE TYPE_BINARY

#define PML_FAX_LOCAL_PHONE_NUM_OID "1.1.3.8"
#define PML_FAX_LOCAL_PHONE_NUM_TYPE TYPE_STRING
#define PML_FAX_STATION_NAME_OID "1.1.3.9"
#define PML_FAX_STATION_NAME_TYPE TYPE_STRING

size_t pml_buildEmbeddedPMLSetPacket(char *buf, size_t size, struct pml *pml, int value);
size_t pml_buildPMLSetPacket(char *buf, size_t size, struct pml *pml, int value);
size_t pml_buildEmbeddedPML(char *buf, size_t size, struct pml *pml, int value, int style);
size_t pml_ConvertToPMLDataFormat(const char *value, size_t len, int type, char *out, size_t size);
size_t pml_ConvertFromPMLDataFormat(const char *data, size_t len, int type, char *out, size_t size);
void pml_PMLToSNMP(const char *oid, char *out, size_t size);

#endif
