/*
 * This file is part of hp-utils.
 * Copyright (C) 2008 Michael Denk
 *
 * hp-utils is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * hp-utils is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with hp-utils; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Michael Denk
 */

#include <time.h>
#include <string.h>

#include "pmlfax.h"
#include "logger.h"
#include "device.h"
#include "pml.h"

int pmlfax_setDateAndTime(struct device *d)
{
	time_t rawtime;
	struct tm *timeinfo;
	char buf[7];
	struct pml pml = { PML_DATE_AND_TIME_OID, PML_DATE_AND_TIME_TYPE };

	time(&rawtime);
	timeinfo = localtime(&rawtime);

	log_debug("Local time: %s", asctime(timeinfo));

	buf[0] = timeinfo->tm_year - 100;
	buf[1] = timeinfo->tm_mon + 1;
	buf[2] = timeinfo->tm_mday;
	buf[3] = timeinfo->tm_wday + 1;
	buf[4] = timeinfo->tm_hour;
	buf[5] = timeinfo->tm_min;
	buf[6] = timeinfo->tm_sec;

	return device_setPML(d, &pml, buf, 7);
}

int pmlfax_getPhoneNum(struct device *d, char *out, size_t size)
{
	struct pml pml = { PML_FAX_LOCAL_PHONE_NUM_OID, PML_FAX_LOCAL_PHONE_NUM_TYPE };
	int ret;

	ret = device_getPML(d, &pml, out, &size);
	out[size] = '\0';
	return ret;
}

int pmlfax_setPhoneNum(struct device *d, const char *s)
{
	struct pml pml = { PML_FAX_LOCAL_PHONE_NUM_OID, PML_FAX_LOCAL_PHONE_NUM_TYPE };

	return device_setPML(d, &pml, s, strlen(s));
}

int pmlfax_getStationName(struct device *d, char *out, size_t size)
{
	struct pml pml = { PML_FAX_STATION_NAME_OID, PML_FAX_STATION_NAME_TYPE };
	int ret;

	ret = device_getPML(d, &pml, out, &size);
	out[size] = '\0';
	return ret;
}

int pmlfax_setStationName(struct device *d, const char *s)
{
	struct pml pml = { PML_FAX_STATION_NAME_OID, PML_FAX_STATION_NAME_TYPE };

	return device_setPML(d, &pml, s, strlen(s));
}
