/*
 * This file is part of hp-utils.
 * Copyright (C) 2008 Michael Denk
 *
 * hp-utils is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * hp-utils is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with hp-utils; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Michael Denk
 */

/* Original copyright notice of base/codes.py from HPLIP */
/* */
/* (c) Copyright 2003-2007 Hewlett-Packard Development Company, L.P. */
/* */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License, or */
/* (at your option) any later version. */
/* */
/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */
/* */
/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA */
/* */
/* Author: Don Welch */
/* */

/* Original copyright notice of base/status.py from HPLIP */
/* */
/* (c) Copyright 2003-2007 Hewlett-Packard Development Company, L.P. */
/* */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License, or */
/* (at your option) any later version. */
/* */
/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */
/* */
/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA */
/* */
/* Author: Don Welch */
/* */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include <ctype.h>

#include "status.h"
#include "logger.h"
#include "util.h"

int NUM_PEN_POS[] = {
	/* STATUS_REV_00 */ 16,
	/* STATUS_REV_01 */ 16,
	/* STATUS_REV_02 */ 16,
	/* STATUS_REV_03 */ 18,
	/* STATUS_REV_04 */ 22
};

int PEN_DATA_SIZE[] = {
	/* STATUS_REV_00 */ 8,
	/* STATUS_REV_01 */ 8,
	/* STATUS_REV_02 */ 4,
	/* STATUS_REV_03 */ 8,
	/* STATUS_REV_04 */ 8
};

int STATUS_POS[] = {
	/* STATUS_REV_00 */ 14,
	/* STATUS_REV_01 */ 14,
	/* STATUS_REV_02 */ 14,
	/* STATUS_REV_03 */ 16,
	/* STATUS_REV_04 */ 20
};

int REVISION_2_TYPE_MAP[] = {
	/* 0 */ AGENT_TYPE_NONE,
	/* 1 */ AGENT_TYPE_BLACK,
	/* 2 */ AGENT_TYPE_CYAN,
	/* 3 */ AGENT_TYPE_MAGENTA,
	/* 4 */ AGENT_TYPE_YELLOW,
	/* 5 */ AGENT_TYPE_BLACK,
	/* 6 */ AGENT_TYPE_CYAN,
	/* 7 */ AGENT_TYPE_MAGENTA,
	/* 8 */ AGENT_TYPE_YELLOW,
};

struct status_block * status_parseSStatus(char *s, char *z);
struct status_block * status_parseVStatus(char *s);

struct status_block * status_parseStatus(char *device_id) {
	const char *SEP = ";";
	char *p;
	char *v = NULL;
	char *s = NULL;
	char *z = NULL;
	struct status_block *status_block = NULL;

	p = strip(strtok(strip(device_id), SEP));
	while (p != NULL) {
		if (strncmp(p, "VSTATUS:", 8) == 0) {
			v = (char *) my_malloc((strlen(p)-8+1) * sizeof *v);
			strcpy(v, p + 8);
			break;
		} else if (strncmp(p, "S:", 2) == 0) {
			s = (char *) my_malloc((strlen(p)-2+1) * sizeof *s);
			strcpy(s, p + 2);
		} else if (strncmp(p, "Z:", 2) == 0) {
			z = (char *) my_malloc((strlen(p)-2+1) * sizeof *z);
			strcpy(z, p + 2);
		}
		
		p = strtok(NULL, SEP);
	}

	log_debug2("VSTATUS: %s", v);
	log_debug2("S: %s", s);
	log_debug2("Z: %s", z);

	if (v != NULL) {
		status_block = status_parseVStatus(v);
	} else if (s != NULL) {
		status_block = status_parseSStatus(s, z);
	}

	my_free(v);
	my_free(s);
	my_free(z);

	return status_block;
}

struct status_block * status_parseSStatus(char *s, char *z)
{
	char *p = NULL;
	int i;
	int *s1 = NULL;
	int s1_size = 0;
	int revision;
	int status_pos, status_byte;
	int c;
	int d;
	int num_pens;
	int pen_data_size;
	int index;
	char *z1s = NULL;
	int *z1 = NULL;
	int z1_size = 0;
	const int Z_SIZE = 6;
	struct status_block *status_block = (struct status_block *) my_malloc(sizeof *status_block);

	if (z != NULL) {
		char * tok = strtok(z, ",");
		do {
			log_debug2("tok: %s", tok);
			if (strlen(tok) > 2 && (strncmp(tok, "05", 2) == 0)) {
				z1s = tok + 2;
				log_debug2("z1s: %s", z1s);

				z1_size = strlen(z1s);
				z1 = (int *) my_malloc(z1_size * sizeof(*z1));
				for (i = 0; i < z1_size; i++) {
					char str[2];
					str[0] = z1s[i];
					str[1] = '\0';
					z1[i] = (int) strtol(str, NULL, 16);
				}
			}
		} while ((tok = strtok(NULL, ",")) != NULL);
	}

	s1_size = strlen(s);
	/* printf("s1_size: %d\n", s1_size); */
	s1 = (int *) my_malloc(s1_size * sizeof(*s1));
	p = s;
	i = 0;
	while (*p != '\0') {
		char str[2] = "\0";
		str[0] = *p;
		s1[i++] = (int) strtol(str, NULL, 16);
		p++;
	}

	revision = s1[1];

	assert(STATUS_REV_00 <= revision && revision <= STATUS_REV_04);

	status_block->revision = revision;

	status_pos = STATUS_POS[revision];
	status_byte = (s1[status_pos]<<4) + s1[status_pos + 1];
	status_block->status_code = status_byte + STATUS_PRINTER_BASE;

	c = NUM_PEN_POS[revision] + 1;
	d = 0;
	num_pens = s1[NUM_PEN_POS[revision]];
	log_debug("Num pens=%d", num_pens);
	index = 0;
	pen_data_size = PEN_DATA_SIZE[revision];
	status_block->num_agents = num_pens;
	status_block->agents = (struct agent *) my_malloc(status_block->num_agents * sizeof *(status_block->agents));

	for (i = 0; i < num_pens; i++) {
		char * buf = (char *) my_malloc((pen_data_size + 1) * sizeof *buf);
		/* pen_data_size <= 8
		 * string parsed as hex
		 * FF => 8 bit
		 * FFFF FFFF => 4 * 8 bit = 32 bit
		 */
		/* uintmax_t info; */
		unsigned long int info;
		struct agent *pen = &(status_block->agents[index]);

		/* TODO remove me */
		/*
		pen->type = -1;
		pen->kind = -1;
		pen->health = -1;
		pen->level_trigger = -1;
		pen->level = -1;
		pen->id = -1;
		*/

		strncpy(buf, s+c, pen_data_size);
		buf[pen_data_size] = '\0';
		/* info = strtoumax(buf, NULL, 16); */
		info = strtoul(buf, NULL, 16);
		/* printf("%" PRIuMAX "\n", info); */
		log_debug2("s[c:c+pen_data_size]=%s, info%d=%lu", buf, i, info);
		my_free(buf);

		if (pen_data_size == 4) {
			int foo = (int) ((info & 0xf000L) >> 12L);
			pen->type = (foo < 0 || foo > 8) ? REVISION_2_TYPE_MAP[foo] : 0;
			if (index < (num_pens / 2)) {
				pen->kind = AGENT_KIND_HEAD;
			} else {
				pen->kind = AGENT_KIND_SUPPLY;
			}
			pen->level_trigger = (int) ((info & 0x0e00L) >> 9L);
			pen->health = (int) ((info & 0x0180L) >> 7L);
			pen->level = (int) (info & 0x007fL);
			pen->id = 0x1f;
		} else if (pen_data_size == 8) {
			pen->kind = (bool(info & 0x80000000L) + ((bool(info & 0x40000000L))<<1L));
			pen->type = (int)((info & 0x3f000000L)>>24L);
			pen->id = (int) ((info & 0xf80000) >> 19L);
			pen->level_trigger = (int) ((info & 0x70000L) >> 16L);
			pen->health = (int) ((info & 0xc000L) >> 14L);
			pen->level = (int) (info & 0xffL);
		} else {
			log_error("Pen data size error");
			pen->kind = AGENT_KIND_NONE;
			pen->type = AGENT_TYPE_NONE;
			pen->id = 0;
			pen->level_trigger = 0;
			pen->health = AGENT_HEALTH_UNKNOWN;
			pen->level = 0;
		}

		if (z1 != NULL) {
			/* TODO Determine cause of IndexError for C6100 (defect #1111) */
			long int dvc;
			int virgin;
			int hp_ink;
			int known;
			int ack;

			if (d+5 < z1_size) {
				char foo[5];
				memset(foo, 0, 5);
				strncpy(foo, z1s+(d+1), 4);
				
				dvc = strtol(foo, NULL, 16);
				virgin = bool(z1[d+5] & 0x8L);
				hp_ink = bool(z1[d+5] & 0x4L);
				known = bool(z1[d+5] & 0x2L);
				ack = bool(z1[d+5] & 0x1L);
			} else {
				dvc = 0;
				virgin = 0;
				hp_ink = 0;
				known = 0;
				ack = 0;
			}
			log_debug2("dvc=%li, virgin=%d, hp_ink=%d, known=%d, ack=%d",
					dvc, virgin, hp_ink, known, ack);
		}

		log_debug2("idx=%d, id=%d, typ=%d, knd=%d, hel=%d, lvl=%d%%, trg=%d",
				index, pen->id, pen->type, pen->kind, pen->health,
				pen->level, pen->level_trigger);

		index++;
		c += pen_data_size;
		d += Z_SIZE;
	}

	return status_block;
}

int vstatus_xlate(char *s)
{
	size_t i;

	for (i = 0; i < strlen(s); i++) {
		s[i] = tolower(s[i]);
	}

	if (strcmp("idle", s) == 0) {
		return STATUS_PRINTER_IDLE;
	} else if (strcmp("busy", s)) {
		return STATUS_PRINTER_BUSY;
	} else if (strcmp("idle", s)) {
		return STATUS_PRINTER_IDLE;
	} else if (strcmp("prnt", s)) {
		return STATUS_PRINTER_PRINTING;
	} else if (strcmp("offf", s)) {
		return STATUS_PRINTER_TURNING_OFF;
	} else if (strcmp("rprt", s)) {
		return STATUS_PRINTER_REPORT_PRINTING;
	} else if (strcmp("cncl", s)) {
		return STATUS_PRINTER_CANCELING;
	} else if (strcmp("iost", s)) {
		return STATUS_PRINTER_IO_STALL;
	} else if (strcmp("dryw", s)) {
		return STATUS_PRINTER_DRY_WAIT_TIME;
	} else if (strcmp("penc", s)) {
		return STATUS_PRINTER_PEN_CHANGE;
	} else if (strcmp("oopa", s)) {
		return STATUS_PRINTER_OUT_OF_PAPER;
	} else if (strcmp("bnej", s)) {
		return STATUS_PRINTER_BANNER_EJECT;
	} else if (strcmp("bnmz", s)) {
		return STATUS_PRINTER_BANNER_MISMATCH;
	} else if (strcmp("phmz", s)) {
		return STATUS_PRINTER_PHOTO_MISMATCH;
	} else if (strcmp("dpmz", s)) {
		return STATUS_PRINTER_DUPLEX_MISMATCH;
	} else if (strcmp("pajm", s)) {
		return STATUS_PRINTER_MEDIA_JAM;
	} else if (strcmp("cars", s)) {
		return STATUS_PRINTER_CARRIAGE_STALL;
	} else if (strcmp("paps", s)) {
		return STATUS_PRINTER_PAPER_STALL;
	} else if (strcmp("penf", s)) {
		return STATUS_PRINTER_PEN_FAILURE;
	} else if (strcmp("erro", s)) {
		return STATUS_PRINTER_HARD_ERROR;
	} else if (strcmp("pwdn", s)) {
		return STATUS_PRINTER_POWER_DOWN;
	} else if (strcmp("fpts", s)) {
		return STATUS_PRINTER_FRONT_PANEL_TEST;
	} else if (strcmp("clno", s)) {
		return STATUS_PRINTER_CLEAN_OUT_TRAY_MISSING;
	}
	return STATUS_PRINTER_IDLE;
}
		
struct status_block * status_parseVStatus(char *s)
{
	struct status_block *status_block = (struct status_block *) my_malloc(sizeof *status_block);
	int c = 0;
	const char *SEP = ",";
	char *p = NULL;
	char *tok;
	int kp = 0;
	int cp = 0;

	tok = strtok(s, SEP);
	if (tok == NULL) {
		log_error("Invalid VSTATUS field in device ID.");
		return NULL;
	}

	p = (char *) my_malloc((strlen(tok) + 1) * sizeof(*tok));
	strcpy(p, tok);
	log_debug2("p: %s", p);

	strtok(NULL, SEP);
	strtok(NULL, SEP);
	tok = strtok(NULL, SEP);
	status_block->status_code = vstatus_xlate(tok);

	strtok(NULL, SEP);
	strtok(NULL, SEP);
	strtok(NULL, SEP);

	while ((tok = strtok(NULL, SEP)) != NULL) {
		log_debug2("while tok: %s", tok);
		if (strncmp(tok, "KP", 2) == 0) {
			kp = atoi(tok+2);
			log_debug2("KP: %d", kp);
		} else if (strncmp(tok, "CP", 2) == 0) {
			cp = atoi(tok+2);
			log_debug2("CP: %d", cp);
		}
	}
	if (strlen(p) == 20) {
		/* TODO
		 * $H00000000$M00000000 style (OJ Pro 1150/70)
		 * need spec
		 */
	} else if (strlen(p) == 8) {
		int i;
		status_block->num_agents = 8/4;
		status_block->agents = (struct agent *) my_malloc(status_block->num_agents * sizeof *(status_block->agents));
		for (i = 0; i < 8; i++) {
			struct agent *pen = &(status_block->agents[i/4]);
			/* TODO remove me */
			/*
			pen->type = -1;
			pen->kind = -1;
			pen->health = -1;
			pen->level_trigger = -1;
			pen->level = -1;
			pen->id = -1;
			*/
			if (c == 0) {
				assert(p[i] == '$');
				c++;
			} else if (c == 1) {
				if (tolower(p[i]) == 'a') {
					pen->type = AGENT_TYPE_NONE;
					pen->kind = AGENT_KIND_NONE;
				}
				c++;
			} else if (c == 2) {
				pen->health = AGENT_HEALTH_OK;
				pen->kind = AGENT_KIND_HEAD_AND_SUPPLY;
				if (tolower(p[i]) == 'b') {
					pen->type = AGENT_TYPE_BLACK;
				} else if (tolower(p[i]) == 'c') {
					pen->type = AGENT_TYPE_CMY;
				} else if (tolower(p[i]) == 'd') {
					pen->type = AGENT_TYPE_KCM;
				} else if (tolower(p[i]) == 'u') {
					pen->type = AGENT_TYPE_NONE;
					pen->health = AGENT_HEALTH_MISINSTALLED;
				}
				c++;
			} else if (c == 3) {
				/*
				if (p == '0') {
					pen->state = 1;
				} else {
					pen->state = 0;
				}
				*/

				pen->level = 0;
				
				if (pen->type == AGENT_TYPE_BLACK) {
					log_debug2("Setting black level to %d", kp);
					pen->level = kp;
				} else if (pen->type == AGENT_TYPE_CMY) {
					log_debug2("Setting cmy level to %d", cp);
					pen->level = cp;
				}

				c = 0;
			}
		}
		my_free(p);
	}

	/* TODO add me... */
	return status_block;
}
