/*
 * This file is part of hp-utils.
 * Copyright (C) 2008 Michael Denk
 *
 * hp-utils is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * hp-utils is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with hp-utils; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Michael Denk
 */

/* Original copyright notice of base/codes.py from HPLIP */
/* */
/* (c) Copyright 2003-2007 Hewlett-Packard Development Company, L.P. */
/* */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License, or */
/* (at your option) any later version. */
/* */
/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */
/* */
/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA */
/* */
/* Author: Don Welch */
/* */

#ifndef _STATUS_H_

#define _STATUS_H_ 1
/* #include "codes.h" */

#define STATUS_REV_00		0x00
#define STATUS_REV_01		0x01
#define STATUS_REV_02		0x02
#define STATUS_REV_03		0x03
#define STATUS_REV_04		0x04
#define STATUS_REV_V		0xff
#define STATUS_REV_UNKNOWN	0xfe

/* */
/* Defines for model query types and status query */
/* */

/* agent info */

/* 'kind' */
#define AGENT_KIND_NONE   0
#define AGENT_KIND_HEAD   1 /* InkJet head (no ink) */
#define AGENT_KIND_SUPPLY   2 /* InkJet supply (ink tank) */
#define AGENT_KIND_HEAD_AND_SUPPLY   3 /* InkJet (cartridge) */
#define AGENT_KIND_TONER_CARTRIDGE   4 /* LaserJet */
#define AGENT_KIND_MAINT_KIT   5 /* LaserJet "Maintenance kit (fuser)" */
#define AGENT_KIND_ADF_KIT   6 /* LaserJet "Document feeder kit" */
#define AGENT_KIND_DRUM_KIT   7 /* LaserJet */
#define AGENT_KIND_TRANSFER_KIT   8 /* LaserJet */
#define AGENT_KIND_INT_BATTERY   9 /* Mobile deskjet (DJ450, etc) */
#define AGENT_KIND_UNKNOWN   0x3e /* (62) */

/* 'type' */
#define AGENT_TYPE_NONE   0
#define AGENT_TYPE_BLACK   1
#define AGENT_TYPE_CMY   2 /* Tricolor */
#define AGENT_TYPE_KCM   3 /* Photo */
#define AGENT_TYPE_CYAN   4
#define AGENT_TYPE_MAGENTA   5
#define AGENT_TYPE_YELLOW   6
#define AGENT_TYPE_CYAN_LOW   7
#define AGENT_TYPE_MAGENTA_LOW   8
#define AGENT_TYPE_YELLOW_LOW   9
#define AGENT_TYPE_GGK   10 /* 2 shades of grey and black */
#define AGENT_TYPE_BLUE   11
#define AGENT_TYPE_KCMY_CM   12 /* K/C/M/Y/LM/LC (02 pen set head) */
#define AGENT_TYPE_LC_LM   13 /* light cyan and light magenta */
/*AGENT_TYPE_Y_M = 14 # yellow and magenta (B9180) */
/*AGENT_TYPE_C_K = 15 # cyan and black (B9180) */
#define AGENT_TYPE_K_Y   14 /* black and yellow (for LJ Pro) */
#define AGENT_TYPE_C_M   15 /* cyan and magenta (for LJ Pro) */
#define AGENT_TYPE_LG_PK   16 /* light grey and photo black */
#define AGENT_TYPE_LG   17 /* light grey */
#define AGENT_TYPE_G   18 /* medium grey */
#define AGENT_TYPE_PG   19 /* photo grey */
#define AGENT_TYPE_WHITE   0x20 /* For ISO 10180 compatibility */
#define AGENT_TYPE_RED   0x21 /* For ISO 10180 compatibility */
#define AGENT_TYPE_UNSPECIFIED = 0x3e /* (62) Used for kind   5, 6, 7, 8, or 9 */
#define AGENT_TYPE_ERROR   0x3f /* (63) */

/* 'health' */
#define AGENT_HEALTH_OK   0
#define AGENT_HEALTH_MISINSTALLED   1 /* supply/cart */
#define AGENT_HEALTH_FAIR_MODERATE   1 /* head */
#define AGENT_HEALTH_INCORRECT   2
#define AGENT_HEALTH_FAILED   3
#define AGENT_HEALTH_OVERTEMP   4 /* Battery */
#define AGENT_HEALTH_CHARGING   5 /* Battery */
#define AGENT_HEALTH_DISCHARGING   6 /* Battery */
#define AGENT_HEALTH_UNKNOWN   0xff

/* 'level' */
#define AGENT_LEVEL_TRIGGER_SUFFICIENT_0   0
#define AGENT_LEVEL_TRIGGER_SUFFICIENT_1   1
#define AGENT_LEVEL_TRIGGER_SUFFICIENT_2   2
#define AGENT_LEVEL_TRIGGER_SUFFICIENT_3   3
#define AGENT_LEVEL_TRIGGER_SUFFICIENT_4   4
#define AGENT_LEVEL_TRIGGER_MAY_BE_LOW   5
#define AGENT_LEVEL_TRIGGER_PROBABLY_OUT   6
#define AGENT_LEVEL_TRIGGER_ALMOST_DEFINITELY_OUT   7

/* Event and status codes */
/* These are used for the 'status-code' returned by DeviceQuery (STATUS_*) */
/* and by the event-code used by Event (EVENT_* + STATUS_*) */

/* If you add a new EVENT/STATUS code, please add the appropriate */
/* entry into the STATUS_TO_ERROR_STATE_MAP */

#define STATUS_UNKNOWN   0

#define EVENT_START_PRINT_JOB   500 /* sent by hp: backend */
#define EVENT_END_PRINT_JOB   501 /* sent by hp: backend */
#define EVENT_START_FAX_PRINT_JOB   600 /* sent by hpfax: backend */
#define EVENT_END_FAX_PRINT_JOB   601 /* sent by hpfax: backend */

#define STATUS_PRINTER_BASE   1000
#define STATUS_PRINTER_IDLE   1000
#define STATUS_PRINTER_BUSY   1001
#define STATUS_PRINTER_PRINTING   1002
#define STATUS_PRINTER_TURNING_OFF   1003
#define STATUS_PRINTER_REPORT_PRINTING   1004
#define STATUS_PRINTER_CANCELING   1005
#define STATUS_PRINTER_IO_STALL   1006
#define STATUS_PRINTER_DRY_WAIT_TIME   1007
#define STATUS_PRINTER_PEN_CHANGE   1008
#define STATUS_PRINTER_OUT_OF_PAPER   1009
#define STATUS_PRINTER_BANNER_EJECT   1010
#define STATUS_PRINTER_BANNER_MISMATCH   1011
#define STATUS_PRINTER_PHOTO_MISMATCH   1012
#define STATUS_PRINTER_DUPLEX_MISMATCH   1013
#define STATUS_PRINTER_MEDIA_JAM   1014
#define STATUS_PRINTER_CARRIAGE_STALL   1015
#define STATUS_PRINTER_PAPER_STALL   1016
#define STATUS_PRINTER_PEN_FAILURE   1017
#define STATUS_PRINTER_HARD_ERROR   1018
#define STATUS_PRINTER_POWER_DOWN   1019
#define STATUS_PRINTER_FRONT_PANEL_TEST   1020
#define STATUS_PRINTER_CLEAN_OUT_TRAY_MISSING   1021
#define STATUS_PRINTER_OUTPUT_BIN_FULL   1022
#define STATUS_PRINTER_MEDIA_SIZE_MISMATCH   1023
#define STATUS_PRINTER_MANUAL_DUPLEX_BLOCK   1024
#define STATUS_PRINTER_SERVCE_STALL   1025
#define STATUS_PRINTER_OUT_OF_INK   1026 /* Also used for out of toner */
#define STATUS_PRINTER_LIO_ERROR   1027
#define STATUS_PRINTER_PUMP_STALL   1028
#define STATUS_PRINTER_TRAY_2_MISSING   1029
#define STATUS_PRINTER_DUPLEXER_MISSING   1030
#define STATUS_PRINTER_REAR_TRAY_MISSING   1031
#define STATUS_PRINTER_PEN_NOT_LATCHED   1032
#define STATUS_PRINTER_BATTERY_VERY_LOW   1033
#define STATUS_PRINTER_SPITTOON_FULL   1034
#define STATUS_PRINTER_OUTPUT_TRAY_CLOSED   1035
#define STATUS_PRINTER_MANUAL_FEED_BLOCKED   1036
#define STATUS_PRINTER_REAR_FEED_BLOCKED   1037
#define STATUS_PRINTER_TRAY_2_OUT_OF_PAPER   1038
#define STATUS_PRINTER_UNABLE_TO_LOAD_FROM_LOCKED_TRAY   1039
#define STATUS_PRINTER_NON_HP_INK   1040
#define STATUS_PRINTER_PEN_CALIBRATION_RESUME   1041
#define STATUS_PRINTER_MEDIA_TYPE_MISMATCH   1042
#define STATUS_PRINTER_CUSTOM_MEDIA_MISMATCH   1043
#define STATUS_PRINTER_PEN_CLEANING   1044
#define STATUS_PRINTER_PEN_CHECKING   1045

struct agent {
	int kind;
	int type;
	int id;
	int health;
	int level;
	int level_trigger;
};
	
struct status_block {
	int revision;
	struct agent *agents;
	int num_agents;
	int top_door;
	int status_code;
	int supply_door;
	int duplexer;
	int photo_tray;
	int in_tray_1;
	int in_tray_2;
	int media_path;
};

struct status_block * status_parseStatus(char *device_id);

#endif
