/*
 * Collection of utility functions and macros
 * Copyright (c) 2008 Michael Denk <code@michaeldenk.de>
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>

#include "util.h"

char * strip(char *s)
{
	char *p1 = s;
	char *p2;

	while (*p1 == ' ' || *p1 == '\t' || *p1 == '\n' || *p1 == '\r') {
		p1++;
	}
	p2 = p1;
	while (*p2 != '\0') {
		p2++;
	}
	if (p2 == p1) {
		return p1;
	}

	p2--;
	while (*p2 == ' ' || *p2 == '\t' || *p2 == '\n' || *p2 == '\r') {
		p2--;
	}
	*(p2+1) = '\0';

	return p1;
}

char * replace(char *str, char from, char to)
{
	char *p;
	for (p = str; *p != '\0'; p++) {
		if (*p == from) {
			*p = to;
		}
	}
	return str;
}

int bool(unsigned long int i)
{
	return (i == 0) ? 0 : 1;
}

void * my_malloc(size_t size)
{
	void *ret = malloc(size);
	if (ret == NULL) {
		fprintf(stderr, "\nMemory error. Out of memory?\n");
		exit(2);
	}
	return ret;
}

void * my_calloc(size_t num, size_t size)
{
	void *ret = calloc(num, size);
	if (ret == NULL) {
		fprintf(stderr, "\nMemory error. Out of memory?\n");
		exit(2);
	}
	return ret;
}

void helpinfo(const char *pname, const char *fmt, ...)
{
	va_list args;

	if (fmt != NULL) {
		va_start(args, fmt);
		fprintf(stderr, "%s: ", pname);
		vfprintf(stderr, fmt, args);
		va_end(args);
		fprintf(stderr, "\n");
	}

	fprintf(stderr, "Try `%s --help' for more information.\n", pname);
	fflush(stderr);
}
